@@ -5,11 +5,12 @@
  *	type.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclListObj.c 1.47 97/08/12 19:02:02
+ * RCS: @(#) $Id: tclListObj.c,v 1.5 1999/04/28 17:06:06 stanton Exp $
  */
 
 #include "tclInt.h"
@@ -146,7 +147,7 @@ Tcl_Obj *
 Tcl_DbNewListObj(objc, objv, file, line)
     int objc;			/* Count of objects referenced by objv. */
     Tcl_Obj *CONST objv[];	/* An array of pointers to Tcl objects. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -185,7 +186,7 @@ Tcl_Obj *
 Tcl_DbNewListObj(objc, objv, file, line)
     int objc;			/* Count of objects referenced by objv. */
     Tcl_Obj *CONST objv[];	/* An array of pointers to Tcl objects. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -235,14 +236,16 @@ Tcl_SetListObj(objPtr, objc, objv)
      * Free any old string rep and any internal rep for the old type.
      */
 
-    Tcl_InvalidateStringRep(objPtr);
     if ((oldTypePtr != NULL) && (oldTypePtr->freeIntRepProc != NULL)) {
 	oldTypePtr->freeIntRepProc(objPtr);
-	objPtr->typePtr = NULL;
     }
+    objPtr->typePtr = NULL;
+    Tcl_InvalidateStringRep(objPtr);
         
     /*
      * Set the object's type to "list" and initialize the internal rep.
+     * However, if there are no elements to put in the list, just give
+     * the object an empty string rep and a NULL type.
      */
 
     if (objc > 0) {
@@ -260,6 +263,8 @@ Tcl_SetListObj(objPtr, objc, objv)
 	
 	objPtr->internalRep.otherValuePtr = (VOID *) listRepPtr;
 	objPtr->typePtr = &tclListType;
+    } else {
+	objPtr->bytes = tclEmptyStringRep;
     }
 }
 
@@ -875,10 +880,10 @@ SetListFromAny(interp, objPtr)
 {
     Tcl_ObjType *oldTypePtr = objPtr->typePtr;
     char *string, *s;
-    const char *elemStart, *nextElem;
+    CONST char *elemStart, *nextElem;
     int lenRemain, length, estCount, elemSize, hasBrace, i, j, result;
     char *limit;		/* Points just after string's last byte. */
-    register const char *p;
+    register CONST char *p;
     register Tcl_Obj **elemPtrs;
     register Tcl_Obj *elemPtr;
     List *listRepPtr;
@@ -887,7 +892,7 @@ SetListFromAny(interp, objPtr)
      * Get the string representation. Make it up-to-date if necessary.
      */
 
-    string = TclGetStringFromObj(objPtr, &length);
+    string = Tcl_GetStringFromObj(objPtr, &length);
 
     /*
      * Parse the string into separate string objects, and create a List
@@ -901,7 +906,7 @@ SetListFromAny(interp, objPtr)
     limit = (string + length);
     estCount = 1;
     for (p = string;  p < limit;  p++) {
-	if (isspace(UCHAR(*p))) {
+	if (isspace(UCHAR(*p))) { /* INTL: ISO space. */
 	    estCount++;
 	}
     }
