@@ -6,33 +6,21 @@
  *	types. These commands are not normally included in Tcl
  *	applications; they're only used for testing.
  *
- * Copyright (c) 1995, 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1998 Sun Microsystems, Inc.
+ * Copyright (c) 1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclTestObj.c,v 1.2 1998/09/14 18:40:02 stanton Exp $
+ * RCS: @(#) $Id: tclTestObj.c,v 1.6 1999/06/15 22:06:17 hershey Exp $
  */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/*
- * Allow this file to be compiled as loadable library (dll) separate
- * from the Tcl shared library (or dll). We don't want all
- * dll symbols to be exported, only the Objtest_Init() function.
- */
-#undef BUILD_tcl
-
 #include "tclInt.h"
 
-#ifdef TCL_USE_STUBS
-Tcl_StubTable *tclStubTable;
-#else
-#define Tcl_Required(a,b,c) Tcl_PkgRequire(a,"Tcl",b,c)
-#endif
-
 /*
  * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the
  * Objtest_Init declaration is in the source file itself, which is only
@@ -60,15 +48,14 @@ static Tcl_Obj *varPtr[NUMBER_OF_OBJECT_VARS];
  * Forward declarations for procedures defined later in this file:
  */
 
-EXTERN Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-
 static int		CheckIfVarUnset _ANSI_ARGS_((Tcl_Interp *interp,
 			    int varIndex));
 static int		GetVariableIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int *indexPtr));
 static void		SetVarToObj _ANSI_ARGS_((int varIndex,
 			    Tcl_Obj *objPtr));
-int			TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 static int		TestbooleanobjCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
@@ -90,18 +77,26 @@ static int		TestobjCmd _ANSI_ARGS_((ClientData dummy,
 static int		TeststringobjCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
+
+typedef struct TestString {
+    int numChars;
+    size_t allocated;
+    size_t uallocated;
+    Tcl_UniChar unicode[2];
+} TestString;
+
 
 /*
  *----------------------------------------------------------------------
  *
- * Tclobjtest_Init --
+ * Objtest_Init, TclObjTest_Init --
  *
  *	This procedure creates additional commands that are used to test the
  *	Tcl object support.
  *
  * Results:
  *	Returns a standard Tcl completion code, and leaves an error
- *	message in interp->result if an error occurs.
+ *	message in the interp's result if an error occurs.
  *
  * Side effects:
  *	Creates and registers several new testing commands.
@@ -110,18 +105,19 @@ static int		TeststringobjCmd _ANSI_ARGS_((ClientData dummy,
  */
 
 int
-#ifdef _USING_PROTOTYPES_
-Objtest_Init (Tcl_Interp *interp)	/* Interpreter for application. */
-#else
+TclObjTest_Init(interp)
+    Tcl_Interp *interp;
+{
+    return Objtest_Init(interp);
+}
+
+int
 Objtest_Init(interp)
     Tcl_Interp *interp;
-#endif
 {
     register int i;
     
-    if (Tcl_Required(interp, TCL_VERSION, 1) == NULL) {
-	return TCL_ERROR;
-    }
+    Tcl_InitStubs(interp, TCL_VERSION, 1);
     for (i = 0;  i < NUMBER_OF_OBJECT_VARS;  i++) {
         varPtr[i] = NULL;
     }
@@ -142,37 +138,6 @@ Objtest_Init(interp)
 	    (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     return Tcl_PkgProvide(interp, "Objtest", TCL_VERSION);
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * TclObjTest_Init --
- *
- *	This procedure creates additional commands that are used to test the
- *	Tcl object support. The same as Tclobjtest_Init().
- *
- * Results:
- *	Returns a standard Tcl completion code, and leaves an error
- *	message in interp->result if an error occurs.
- *
- * Side effects:
- *	Creates and registers several new testing commands.
- *
- *----------------------------------------------------------------------
- */
-
-int
-#ifdef _USING_PROTOTYPES_
-TclObjTest_Init (Tcl_Interp *interp)	/* Interpreter for application. */
-#else
-TclObjTest_Init(interp)
-    Tcl_Interp *interp;
-#endif
-{
-    return Objtest_Init(interp);
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -193,21 +158,13 @@ TclObjTest_Init(interp)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestbooleanobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestbooleanobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestbooleanobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
-    int varIndex, boolValue, length;
+    int varIndex, boolValue;
     char *index, *subCmd;
 
     if (objc < 3) {
@@ -216,16 +173,12 @@ TestbooleanobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * THIS FAILS IF AN OBJECT'S STRING REP HAS A NULL BYTE.
-     */
-
-    index = Tcl_GetStringFromObj(objv[2], &length);
+    index = Tcl_GetString(objv[2]);
     if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
 	return TCL_ERROR;
     }
 
-    subCmd = Tcl_GetStringFromObj(objv[1], &length);
+    subCmd = Tcl_GetString(objv[1]);
     if (strcmp(subCmd, "set") == 0) {
 	if (objc != 4) {
 	    goto wrongNumArgs;
@@ -275,7 +228,7 @@ TestbooleanobjCmd(dummy, interp, objc, objv)
 	Tcl_SetObjResult(interp, varPtr[varIndex]);
     } else {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-		"bad option \"", Tcl_GetStringFromObj(objv[1], (int *) NULL),
+		"bad option \"", Tcl_GetString(objv[1]),
 		"\": must be set, get, or not", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -300,21 +253,12 @@ TestbooleanobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestconvertobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestconvertobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestconvertobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
-    int length;
     char *subCmd;
     char buf[20];
 
@@ -324,11 +268,7 @@ TestconvertobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * THIS FAILS IF AN OBJECT'S STRING REP HAS A NULL BYTE.
-     */
-
-    subCmd = Tcl_GetStringFromObj(objv[1], &length);
+    subCmd = Tcl_GetString(objv[1]);
     if (strcmp(subCmd, "double") == 0) {
 	double d;
 
@@ -342,7 +282,7 @@ TestconvertobjCmd(dummy, interp, objc, objv)
         Tcl_AppendToObj(Tcl_GetObjResult(interp), buf, -1);
     } else {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-		"bad option \"", Tcl_GetStringFromObj(objv[1], (int *) NULL),
+		"bad option \"", Tcl_GetString(objv[1]),
 		"\": must be double", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -369,21 +309,13 @@ TestconvertobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestdoubleobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestdoubleobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestdoubleobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
-    int varIndex, length;
+    int varIndex;
     double doubleValue;
     char *index, *subCmd, *string;
 	
@@ -393,21 +325,17 @@ TestdoubleobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * THIS FAILS IF AN OBJECT'S STRING REP HAS A NULL BYTE.
-     */
-
-    index = Tcl_GetStringFromObj(objv[2], &length);
+    index = Tcl_GetString(objv[2]);
     if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
 	return TCL_ERROR;
     }
 
-    subCmd = Tcl_GetStringFromObj(objv[1], &length);
+    subCmd = Tcl_GetString(objv[1]);
     if (strcmp(subCmd, "set") == 0) {
 	if (objc != 4) {
 	    goto wrongNumArgs;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
 	if (Tcl_GetDouble(interp, string, &doubleValue) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -470,7 +398,7 @@ TestdoubleobjCmd(dummy, interp, objc, objv)
 	Tcl_SetObjResult(interp, varPtr[varIndex]);
     } else {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-		"bad option \"", Tcl_GetStringFromObj(objv[1], (int *) NULL),
+		"bad option \"", Tcl_GetString(objv[1]),
 		"\": must be set, get, mult10, or div10", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -496,25 +424,17 @@ TestdoubleobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestindexobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestindexobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestindexobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
-    int allowAbbrev, index, index2, setError, i, length, result;
+    int allowAbbrev, index, index2, setError, i, result;
     char **argv;
     static char *tablePtr[] = {"a", "b", "check", (char *) NULL};
 
-    if ((objc == 3) && (strcmp(Tcl_GetStringFromObj(objv[1], &length),
+    if ((objc == 3) && (strcmp(Tcl_GetString(objv[1]),
 	    "check") == 0)) {
 	/*
 	 * This code checks to be sure that the results of
@@ -547,13 +467,27 @@ TestindexobjCmd(dummy, interp, objc, objv)
     if (Tcl_GetBooleanFromObj(interp, objv[2], &allowAbbrev) != TCL_OK) {
 	return TCL_ERROR;
     }
+
     argv = (char **) ckalloc((unsigned) ((objc-3) * sizeof(char *)));
     for (i = 4; i < objc; i++) {
-	argv[i-4] = Tcl_GetStringFromObj(objv[i], &length);
+	argv[i-4] = Tcl_GetString(objv[i]);
     }
     argv[objc-4] = NULL;
-    result = Tcl_GetIndexFromObj(setError ? interp : (Tcl_Interp *) NULL, objv[3],
-	    argv, "token", allowAbbrev ? 0 : TCL_EXACT, &index);
+    
+    /*
+     * Tcl_GetIndexFromObj assumes that the table is statically-allocated
+     * so that its address is different for each index object. If we
+     * accidently allocate a table at the same address as that cached in
+     * the index object, clear out the object's cached state.
+     */
+
+    if ((objv[3]->typePtr == Tcl_GetObjType("index"))
+	    && (objv[3]->internalRep.twoPtrValue.ptr1 == (VOID *) argv)) {
+	objv[3]->typePtr = NULL;
+    }
+
+    result = Tcl_GetIndexFromObj((setError? interp : NULL), objv[3],
+	    argv, "token", (allowAbbrev? 0 : TCL_EXACT), &index);
     ckfree((char *) argv);
     if (result == TCL_OK) {
 	Tcl_SetIntObj(Tcl_GetObjResult(interp), index);
@@ -580,21 +514,13 @@ TestindexobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestintobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestintobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestintobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
-    int intValue, varIndex, length, i;
+    int intValue, varIndex, i;
     long longValue;
     char *index, *subCmd, *string;
 	
@@ -604,21 +530,17 @@ TestintobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * THIS FAILS IF AN OBJECT'S STRING REP HAS A NULL BYTE.
-     */
-
-    index = Tcl_GetStringFromObj(objv[2], &length);
+    index = Tcl_GetString(objv[2]);
     if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
 	return TCL_ERROR;
     }
 
-    subCmd = Tcl_GetStringFromObj(objv[1], &length);
+    subCmd = Tcl_GetString(objv[1]);
     if (strcmp(subCmd, "set") == 0) {
 	if (objc != 4) {
 	    goto wrongNumArgs;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
 	if (Tcl_GetInt(interp, string, &i) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -642,7 +564,7 @@ TestintobjCmd(dummy, interp, objc, objv)
 	if (objc != 4) {
 	    goto wrongNumArgs;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
 	if (Tcl_GetInt(interp, string, &i) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -656,7 +578,7 @@ TestintobjCmd(dummy, interp, objc, objv)
 	if (objc != 4) {
 	    goto wrongNumArgs;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
 	if (Tcl_GetInt(interp, string, &i) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -697,6 +619,15 @@ TestintobjCmd(dummy, interp, objc, objv)
 	    return TCL_ERROR;
 	}
 	Tcl_SetObjResult(interp, varPtr[varIndex]);
+    } else if (strcmp(subCmd, "get2") == 0) {
+	if (objc != 3) {
+	    goto wrongNumArgs;
+	}
+	if (CheckIfVarUnset(interp, varIndex)) {
+	    return TCL_ERROR;
+	}
+	string = Tcl_GetString(varPtr[varIndex]);
+	Tcl_AppendToObj(Tcl_GetObjResult(interp), string, -1);
     } else if (strcmp(subCmd, "inttoobigtest") == 0) {
 	/*
 	 * If long ints have more bits than ints on this platform, verify
@@ -705,26 +636,24 @@ TestintobjCmd(dummy, interp, objc, objv)
 	 * to fit in an int.
 	 */
 	
-	long maxLong = LONG_MAX;
-	
 	if (objc != 3) {
 	    goto wrongNumArgs;
 	}
-	if (INT_MAX == LONG_MAX) { /* int is same size as long int */
-	    Tcl_AppendToObj(Tcl_GetObjResult(interp), "1", -1);
+#if (INT_MAX == LONG_MAX)   /* int is same size as long int */
+	Tcl_AppendToObj(Tcl_GetObjResult(interp), "1", -1);
+#else 
+	if ((varPtr[varIndex] != NULL) && !Tcl_IsShared(varPtr[varIndex])) {
+	    Tcl_SetLongObj(varPtr[varIndex], LONG_MAX);
 	} else {
-	    if ((varPtr[varIndex] != NULL) && !Tcl_IsShared(varPtr[varIndex])) {
-		Tcl_SetLongObj(varPtr[varIndex], maxLong);
-	    } else {
-		SetVarToObj(varIndex, Tcl_NewLongObj(maxLong));
-	    }
-	    if (Tcl_GetIntFromObj(interp, varPtr[varIndex], &i) != TCL_OK) {
-		Tcl_ResetResult(interp);
-		Tcl_AppendToObj(Tcl_GetObjResult(interp), "1", -1);
-		return TCL_OK;
-	    }
-	    Tcl_AppendToObj(Tcl_GetObjResult(interp), "0", -1);
+	    SetVarToObj(varIndex, Tcl_NewLongObj(LONG_MAX));
 	}
+	if (Tcl_GetIntFromObj(interp, varPtr[varIndex], &i) != TCL_OK) {
+	    Tcl_ResetResult(interp);
+	    Tcl_AppendToObj(Tcl_GetObjResult(interp), "1", -1);
+	    return TCL_OK;
+	}
+	Tcl_AppendToObj(Tcl_GetObjResult(interp), "0", -1);
+#endif
     } else if (strcmp(subCmd, "mult10") == 0) {
 	if (objc != 3) {
 	    goto wrongNumArgs;
@@ -761,8 +690,9 @@ TestintobjCmd(dummy, interp, objc, objv)
 	Tcl_SetObjResult(interp, varPtr[varIndex]);
     } else {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-		"bad option \"", Tcl_GetStringFromObj(objv[1], (int *) NULL),
-		"\": must be set, get, mult10, or div10", (char *) NULL);
+		"bad option \"", Tcl_GetString(objv[1]),
+		"\": must be set, get, get2, mult10, or div10",
+		(char *) NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -786,25 +716,15 @@ TestintobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TestobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TestobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
     int varIndex, destIndex, i;
     char *index, *subCmd, *string;
     Tcl_ObjType *targetType;
-    char buf[20];
-    int length;
 	
     if (objc < 2) {
 	wrongNumArgs:
@@ -812,23 +732,19 @@ TestobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * THIS FAILS IF AN OBJECT'S STRING REP HAS A NULL BYTE.
-     */
-
-    subCmd = Tcl_GetStringFromObj(objv[1], &length);
+    subCmd = Tcl_GetString(objv[1]);
     if (strcmp(subCmd, "assign") == 0) {
         if (objc != 4) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
         if (CheckIfVarUnset(interp, varIndex)) {
 	    return TCL_ERROR;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
         if (GetVariableIndex(interp, string, &destIndex) != TCL_OK) {
             return TCL_ERROR;
         }
@@ -839,14 +755,14 @@ TestobjCmd(dummy, interp, objc, objv)
         if (objc != 4) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
         if (CheckIfVarUnset(interp, varIndex)) {
 	    return TCL_ERROR;
 	}
-        typeName = Tcl_GetStringFromObj(objv[3], &length);
+        typeName = Tcl_GetString(objv[3]);
         if ((targetType = Tcl_GetObjType(typeName)) == NULL) {
 	    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
 		    "no type ", typeName, " found", (char *) NULL);
@@ -861,14 +777,14 @@ TestobjCmd(dummy, interp, objc, objv)
         if (objc != 4) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
         if (CheckIfVarUnset(interp, varIndex)) {
 	    return TCL_ERROR;
 	}
-	string = Tcl_GetStringFromObj(objv[3], &length);
+	string = Tcl_GetString(objv[3]);
         if (GetVariableIndex(interp, string, &destIndex) != TCL_OK) {
             return TCL_ERROR;
         }
@@ -888,30 +804,49 @@ TestobjCmd(dummy, interp, objc, objv)
         if (objc != 3) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
         SetVarToObj(varIndex, Tcl_NewObj());
 	Tcl_SetObjResult(interp, varPtr[varIndex]);
-    } else if (strcmp(subCmd, "refcount") == 0) {
+    } else if (strcmp(subCmd, "objtype") == 0) {
+	char *typeName;
+
+	/*
+	 * return an object containing the name of the argument's type
+	 * of internal rep.  If none exists, return "none".
+	 */
+	
         if (objc != 3) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+	if (objv[2]->typePtr == NULL) {
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj("none", -1));
+	} else {
+	    typeName = objv[2]->typePtr->name;
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(typeName, -1));
+	}
+    } else if (strcmp(subCmd, "refcount") == 0) {
+	char buf[TCL_INTEGER_SPACE];
+
+        if (objc != 3) {
+            goto wrongNumArgs;
+        }
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
         if (CheckIfVarUnset(interp, varIndex)) {
 	    return TCL_ERROR;
 	}
-        sprintf(buf, "%d", varPtr[varIndex]->refCount);
-        Tcl_AppendToObj(Tcl_GetObjResult(interp), buf, -1);
+	TclFormatInt(buf, varPtr[varIndex]->refCount);
+        Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if (strcmp(subCmd, "type") == 0) {
         if (objc != 3) {
             goto wrongNumArgs;
         }
-        index = Tcl_GetStringFromObj(objv[2], &length);
+        index = Tcl_GetString(objv[2]);
         if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
             return TCL_ERROR;
         }
@@ -928,15 +863,16 @@ TestobjCmd(dummy, interp, objc, objv)
         if (objc != 2) {
             goto wrongNumArgs;
         }
-	if (Tcl_AppendAllObjTypes(interp, Tcl_GetObjResult(interp)) != TCL_OK) {
+	if (Tcl_AppendAllObjTypes(interp,
+		Tcl_GetObjResult(interp)) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     } else {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
 		"bad option \"",
-		Tcl_GetStringFromObj(objv[1], (int *) NULL),
+		Tcl_GetString(objv[1]),
 		"\": must be assign, convert, duplicate, freeallvars, ",
-		"newobj, objcount, refcount, type, or types",
+		"newobj, objcount, objtype, refcount, type, or types",
 		(char *) NULL);
 	return TCL_ERROR;
     }
@@ -962,26 +898,19 @@ TestobjCmd(dummy, interp, objc, objv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TeststringobjCmd (
-    ClientData dummy,		/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int objc,			/* Number of arguments. */
-    Tcl_Obj *CONST objv[])	/* Argument objects. */
-#else
-TeststringobjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
+TeststringobjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
-#endif
 {
     int varIndex, option, i, length;
-#define MAX_STRINGS 10
+#define MAX_STRINGS 11
     char *index, *string, *strings[MAX_STRINGS+1];
+    TestString *strPtr;
     static char *options[] = {
-	"append", "appendstrings", "get", "length", "length2",
-	"set", "set2", "setlength", (char *) NULL
+	"append", "appendstrings", "get", "get2", "length", "length2",
+	"set", "set2", "setlength", "ualloc", (char *) NULL
     };
 
     if (objc < 3) {
@@ -990,7 +919,7 @@ TeststringobjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    index = Tcl_GetStringFromObj(objv[2], (int *) NULL);
+    index = Tcl_GetString(objv[2]);
     if (GetVariableIndex(interp, index, &varIndex) != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -1019,7 +948,7 @@ TeststringobjCmd(dummy, interp, objc, objv)
 	    if (Tcl_IsShared(varPtr[varIndex])) {
 		SetVarToObj(varIndex, Tcl_DuplicateObj(varPtr[varIndex]));
 	    }
-	    string = Tcl_GetStringFromObj(objv[3], (int *) NULL);
+	    string = Tcl_GetString(objv[3]);
 	    Tcl_AppendToObj(varPtr[varIndex], string, length);
 	    Tcl_SetObjResult(interp, varPtr[varIndex]);
 	    break;
@@ -1040,9 +969,11 @@ TeststringobjCmd(dummy, interp, objc, objv)
 		SetVarToObj(varIndex, Tcl_DuplicateObj(varPtr[varIndex]));
 	    }
 	    for (i = 3;  i < objc;  i++) {
-		strings[i-3] = Tcl_GetStringFromObj(objv[i], (int *) NULL);
+		strings[i-3] = Tcl_GetString(objv[i]);
+	    }
+	    for ( ; i < 12 + 3; i++) {
+		strings[i - 3] = NULL;
 	    }
-	    strings[objc-3] = NULL;
 	    Tcl_AppendStringsToObj(varPtr[varIndex], strings[0], strings[1],
 		    strings[2], strings[3], strings[4], strings[5],
 		    strings[6], strings[7], strings[8], strings[9],
@@ -1058,21 +989,37 @@ TeststringobjCmd(dummy, interp, objc, objv)
 	    }
 	    Tcl_SetObjResult(interp, varPtr[varIndex]);
 	    break;
-	case 3:				/* length */
+	case 3:				/* get2 */
+	    if (objc != 3) {
+		goto wrongNumArgs;
+	    }
+	    if (CheckIfVarUnset(interp, varIndex)) {
+		return TCL_ERROR;
+	    }
+	    string = Tcl_GetString(varPtr[varIndex]);
+	    Tcl_AppendToObj(Tcl_GetObjResult(interp), string, -1);
+	    break;
+	case 4:				/* length */
 	    if (objc != 3) {
 		goto wrongNumArgs;
 	    }
 	    Tcl_SetIntObj(Tcl_GetObjResult(interp), (varPtr[varIndex] != NULL)
 		    ? varPtr[varIndex]->length : -1);
 	    break;
-	case 4:				/* length2 */
+	case 5:				/* length2 */
 	    if (objc != 3) {
 		goto wrongNumArgs;
 	    }
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), (varPtr[varIndex] != NULL)
-		    ? (int) varPtr[varIndex]->internalRep.longValue : -1);
+	    if (varPtr[varIndex] != NULL) {
+		strPtr = (TestString *)
+		    (varPtr[varIndex])->internalRep.otherValuePtr;
+		length = (int) strPtr->allocated;
+	    } else {
+		length = -1;
+	    }
+	    Tcl_SetIntObj(Tcl_GetObjResult(interp), length);
 	    break;
-	case 5:				/* set */
+	case 6:				/* set */
 	    if (objc != 4) {
 		goto wrongNumArgs;
 	    }
@@ -1095,13 +1042,13 @@ TeststringobjCmd(dummy, interp, objc, objv)
 	    }
 	    Tcl_SetObjResult(interp, varPtr[varIndex]);
 	    break;
-	case 6:				/* set2 */
+	case 7:				/* set2 */
 	    if (objc != 4) {
 		goto wrongNumArgs;
 	    }
 	    SetVarToObj(varIndex, objv[3]);
 	    break;
-	case 7:				/* setlength */
+	case 8:				/* setlength */
 	    if (objc != 4) {
 		goto wrongNumArgs;
 	    }
@@ -1112,6 +1059,19 @@ TeststringobjCmd(dummy, interp, objc, objv)
 		Tcl_SetObjLength(varPtr[varIndex], length);
 	    }
 	    break;
+	case 9:				/* ualloc */
+	    if (objc != 3) {
+		goto wrongNumArgs;
+	    }
+	    if (varPtr[varIndex] != NULL) {
+		strPtr = (TestString *)
+		    (varPtr[varIndex])->internalRep.otherValuePtr;
+		length = (int) strPtr->uallocated;
+	    } else {
+		length = -1;
+	    }
+	    Tcl_SetIntObj(Tcl_GetObjResult(interp), length);
+	    break;
     }
 
     return TCL_OK;
@@ -1137,15 +1097,9 @@ TeststringobjCmd(dummy, interp, objc, objv)
  */
 
 static void
-#ifdef _USING_PROTOTYPES_
-SetVarToObj (
-    int varIndex,		/* Designates the assignment variable. */
-    Tcl_Obj *objPtr)		/* Points to object to assign to var. */
-#else
 SetVarToObj(varIndex, objPtr)
     int varIndex;		/* Designates the assignment variable. */
     Tcl_Obj *objPtr;		/* Points to object to assign to var. */
-#endif
 {
     if (varPtr[varIndex] != NULL) {
 	Tcl_DecrRefCount(varPtr[varIndex]);
@@ -1173,21 +1127,12 @@ SetVarToObj(varIndex, objPtr)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-GetVariableIndex (
-    Tcl_Interp *interp,         /* Interpreter for error reporting. */
-    char *string,               /* String containing a variable index
-				 * specified as a nonnegative number less
-				 * than NUMBER_OF_OBJECT_VARS. */
-    int *indexPtr)              /* Place to store converted result. */
-#else
 GetVariableIndex(interp, string, indexPtr)
     Tcl_Interp *interp;         /* Interpreter for error reporting. */
     char *string;               /* String containing a variable index
 				 * specified as a nonnegative number less
 				 * than NUMBER_OF_OBJECT_VARS. */
     int *indexPtr;              /* Place to store converted result. */
-#endif
 {
     int index;
     
@@ -1223,18 +1168,12 @@ GetVariableIndex(interp, string, indexPtr)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-CheckIfVarUnset (
-    Tcl_Interp *interp,		/* Interpreter for error reporting. */
-    int varIndex)		/* Index of the test variable to check. */
-#else
 CheckIfVarUnset(interp, varIndex)
     Tcl_Interp *interp;		/* Interpreter for error reporting. */
     int varIndex;		/* Index of the test variable to check. */
-#endif
 {
     if (varPtr[varIndex] == NULL) {
-	char buf[100];
+	char buf[32 + TCL_INTEGER_SPACE];
 	
 	sprintf(buf, "variable %d is unset (NULL)", varIndex);
 	Tcl_ResetResult(interp);
