@@ -6,11 +6,12 @@
  *
  * Copyright (c) 1993 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclAppInit.c,v 1.4 1999/02/03 02:58:26 stanton Exp $
+ * RCS: @(#) $Id: tclAppInit.c,v 1.5 1999/04/16 00:48:03 stanton Exp $
  */
 
 #ifdef TCL_XT_TEST
@@ -33,20 +34,27 @@ int *tclDummyMathPtr = (int *) matherr;
 
 
 #ifdef TCL_TEST
-extern int		Procbodytest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-extern int		Procbodytest_SafeInit _ANSI_ARGS_((Tcl_Interp *interp));
-extern int		ObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-extern int		Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-extern int		Unixtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-#endif /* TCL_TEST */
-#ifdef TCL_XT_TEST
-extern int		Xttest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-#endif
+
+#include "tclInt.h"
 
 #ifdef	__cplusplus
 }
 #endif
 
+extern int		Proctest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Proctest_SafeInit _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Unixtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+#ifdef TCL_THREADS
+extern int		Threadtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+#endif
+
+#endif /* TCL_TEST */
+
+#ifdef TCL_XT_TEST
+extern int		Xttest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -75,9 +83,16 @@ main(argc, argv)
     char **argv;		/* Values of command-line arguments. */
 #endif
 {
-#ifdef TCL_XT_TEST
-    XtToolkitInitialize();
+#ifdef TCL_TEST
+    /*
+     * Pass the build time location of the tcl library (to find init.tcl)
+     */
+    Tcl_Obj *path;
+    path = Tcl_NewStringObj(TCL_BUILDTIME_LIBRARY, -1);
+    TclSetLibraryPath(Tcl_NewListObj(1,&path));
+
 #endif
+
     Tcl_Main(argc, argv, Tcl_AppInit);
     return 0;			/* Needed only to prevent compiler warning. */
 }
@@ -93,7 +108,7 @@ main(argc, argv)
  *
  * Results:
  *	Returns a standard Tcl completion code, and leaves an error
- *	message in interp->result if an error occurs.
+ *	message in the interp's result if an error occurs.
  *
  * Side effects:
  *	Depends on the startup script.
@@ -115,9 +130,9 @@ Tcl_AppInit(interp)
 
 #ifdef TCL_TEST
 #ifdef TCL_XT_TEST
-     if (Xttest_Init(interp) == TCL_ERROR) {
-	 return TCL_ERROR;
-     }
+    if (Xttest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
     Tcl_StaticPackage(interp, "Xttest", Xttest_Init,
 	    (Tcl_PackageInitProc *) NULL);
 #endif
@@ -136,11 +151,18 @@ Tcl_AppInit(interp)
     }
     Tcl_StaticPackage(interp, "Unixtest", Unixtest_Init,
             (Tcl_PackageInitProc *) NULL);
-    if (Procbodytest_Init(interp) == TCL_ERROR) {
+#ifdef TCL_THREADS
+    if (Threadtest_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
-    Tcl_StaticPackage(interp, "procbodytest", Procbodytest_Init,
-            Procbodytest_SafeInit);
+    Tcl_StaticPackage(interp, "Threadtest", Threadtest_Init,
+            (Tcl_PackageInitProc *) NULL);
+#endif
+    if (Proctest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    Tcl_StaticPackage(interp, "Proctest", Proctest_Init,
+            Proctest_SafeInit);
 #endif /* TCL_TEST */
 
     /*
