@@ -8,48 +8,26 @@
 #include "tclWinInt.h"
 
 /*
- * The following declaration is for the VC++ DLL entry point.
+ * VC++ has an alternate entry point called DllMain, so we need to rename
+ * our entry point.
  */
 
-BOOL APIENTRY		DllMain _ANSI_ARGS_((HINSTANCE hInst,
-			    DWORD reason, LPVOID reserved));
+#if defined(_MSC_VER) || defined(__GNUC__)
+#  define DllEntryPoint DllMain
+#endif
+
 static int		tclProcessesAttached = 0;
+
 
 /*
  *----------------------------------------------------------------------
  *
  * DllEntryPoint --
  *
- *	This wrapper function is used by Borland to invoke the
- *	initialization code for Tcl.  It simply calls the DllMain
- *	routine.
- *
- * Results:
- *	See DllMain.
- *
- * Side effects:
- *	See DllMain.
- *
- *----------------------------------------------------------------------
- */
-
-BOOL APIENTRY
-DllEntryPoint(hInst, reason, reserved)
-    HINSTANCE hInst;		/* Library instance handle. */
-    DWORD reason;		/* Reason this function is being called. */
-    LPVOID reserved;		/* Not used. */
-{
-    return DllMain(hInst, reason, reserved);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * DllMain --
- *
  *	This routine is called by the VC++ C run time library init
- *	code, or the DllEntryPoint routine.  It is responsible for
- *	initializing various dynamically loaded libraries.
+ *	code, or the Borland initialization routine.  It is responsible
+ *	for initializing various dynamically loaded libraries.
+ *	With VC++ and GNUC, it is renamed to DllMain
  *
  * Results:
  *	TRUE on sucess, FALSE on failure.
@@ -59,8 +37,9 @@ DllEntryPoint(hInst, reason, reserved)
  *
  *----------------------------------------------------------------------
  */
+
 BOOL APIENTRY
-DllMain(hInst, reason, reserved)
+DllEntryPoint(hInst, reason, reserved)
     HINSTANCE hInst;		/* Library instance handle. */
     DWORD reason;		/* Reason this function is being called. */
     LPVOID reserved;		/* Not used. */
