@@ -8,24 +8,10 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinTest.c,v 1.2 1998/09/14 18:40:20 stanton Exp $
+ * RCS: @(#) $Id: tclWinTest.c,v 1.3 1999/04/16 00:48:10 stanton Exp $
  */
 
-/*
- * Allow this file to be compiled as loadable library (dll) separate
- * from the Tcl shared library (or dll). We don't want all
- * dll symbols to be exported, only the Wintest_Init() function.
- */
-#undef BUILD_tcl
-
-#include "tclInt.h"
-#include "tclPort.h"
-
-#ifdef TCL_USE_STUBS
-Tcl_StubTable *tclStubTable;
-#else
-#define Tcl_Required(a,b,c) Tcl_PkgRequired(a,"Tcl",b,c)
-#endif
+#include "tclWinInt.h"
 
 /*
  * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the
@@ -39,7 +25,6 @@ Tcl_StubTable *tclStubTable;
 /*
  * Forward declarations of procedures defined later in this file:
  */
-
 EXTERN int		Wintest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 int			TclplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
 static int		TesteventloopCmd _ANSI_ARGS_((ClientData dummy,
@@ -71,7 +56,7 @@ Wintest_Init(interp)
     Tcl_Interp *interp;		/* Interpreter to add commands to. */
 #endif
 {
-    if (Tcl_Required(interp, TCL_VERSION, 0) == NULL) {
+    if (Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
 	return TCL_ERROR;
     }
     if (Tcl_PkgProvide(interp, "Wintest", TCL_VERSION) != TCL_OK) {
